package java.awt.image;

import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import sun.awt.image.ImagingLib;
import java.util.Arrays;
import java.awt.RenderingHints;

public class BandCombineOp implements RasterOp
{
    float[][] matrix;
    int nrows;
    int ncols;
    RenderingHints hints;
    
    public BandCombineOp(float[][] array, RenderingHints hints) {
        this.nrows = 0;
        this.ncols = 0;
        this.nrows = array.length;
        this.ncols = array[0].length;
        this.matrix = new float[this.nrows][];
        int i = 0;
        while (i < this.nrows) {
            if (this.ncols > array[i].length) {
                throw new IndexOutOfBoundsException("row " + i + " too short");
            }
            this.matrix[i] = Arrays.copyOf(array[i], this.ncols);
            ++i;
        }
        this.hints = hints;
    }
    
    public final float[][] getMatrix() {
        float[][] array = new float[this.nrows][];
        for (int i = 0; i < this.nrows; ++i) {
            array[i] = Arrays.copyOf(this.matrix[i], this.ncols);
        }
        return array;
    }
    
    @Override
    public WritableRaster filter(Raster src, WritableRaster compatibleDestRaster) {
        int numBands = src.getNumBands();
        if (this.ncols != numBands && this.ncols != numBands + 1) {
            throw new IllegalArgumentException("Number of columns in the matrix (" + this.ncols + ") must be equal to the number of bands ([+1]) in src (" + numBands + ").");
        }
        if (compatibleDestRaster == null) {
            compatibleDestRaster = this.createCompatibleDestRaster(src);
        }
        else if (this.nrows != compatibleDestRaster.getNumBands()) {
            throw new IllegalArgumentException("Number of rows in the matrix (" + this.nrows + ") must be equal to the number of bands ([+1]) in dst (" + numBands + ").");
        }
        if (ImagingLib.filter(this, src, compatibleDestRaster) != null) {
            return compatibleDestRaster;
        }
        int[] array = null;
        int[] array2 = new int[compatibleDestRaster.getNumBands()];
        int minX = src.getMinX();
        int minY = src.getMinY();
        int minX2 = compatibleDestRaster.getMinX();
        int minY2 = compatibleDestRaster.getMinY();
        if (this.ncols == numBands) {
            for (int n2 = 0; n2 < src.getHeight(); ++n2, ++minY, ++minY2) {
                for (int n3 = 0; n3 < src.getWidth(); ++n3, ++minX, ++minX2) {
                    array = src.getPixel(minX, minY, array);
                    float n5;
                    for (int n4 = 0; n4 < this.nrows; array2[n4] = (int)n5, ++n4) {
                        n5 = 0.0f;
                        for (int n6 = 0; n6 < this.ncols; ++n6) {
                            n5 += this.matrix[n4][n6] * (float)array[n6];
                        }
                    }
                    compatibleDestRaster.setPixel(minX2, minY2, array2);
                }
            }
            return compatibleDestRaster;
        }
        for (int i = 0; i < src.getHeight(); ++i, ++minY, ++minY2) {
            for (int j = 0; j < src.getWidth(); ++j, ++minX, ++minX2) {
                array = src.getPixel(minX, minY, array);
                float n;
                for (int k = 0; k < this.nrows; array2[k] = (int)(n + this.matrix[k][numBands]), ++k) {
                    n = 0.0f;
                    for (int l = 0; l < numBands; ++l) {
                        n += this.matrix[k][l] * (float)array[l];
                    }
                }
                compatibleDestRaster.setPixel(minX2, minY2, array2);
            }
        }
        return compatibleDestRaster;
    }
    
    @Override
    public final Rectangle2D getBounds2D(Raster raster) {
        return raster.getBounds();
    }
    
    @Override
    public WritableRaster createCompatibleDestRaster(Raster raster) {
        int numBands = raster.getNumBands();
        if (this.ncols != numBands && this.ncols != numBands + 1) {
            throw new IllegalArgumentException("Number of columns in the matrix (" + this.ncols + ") must be equal to the number of bands ([+1]) in src (" + numBands + ").");
        }
        if (raster.getNumBands() == this.nrows) {
            return raster.createCompatibleWritableRaster();
        }
        throw new IllegalArgumentException("Don't know how to create a  compatible Raster with " + this.nrows + " bands.");
    }
    
    @Override
    public final Point2D getPoint2D(Point2D point2D, Point2D point2D2) {
        if (point2D2 == null) {
            point2D2 = new Point2D.Float();
        }
        point2D2.setLocation(point2D.getX(), point2D.getY());
        return point2D2;
    }
    
    @Override
    public final RenderingHints getRenderingHints() {
        return this.hints;
    }
}
